//
//  HeliumSdk.h
//  HeliumSdk
//
//  Created by Geoff Heeren on 12/10/18.
//

#import <Foundation/Foundation.h>
#import <HeliumSdk/CHBHPartnerClientProtocol.h>
#import <HeliumSdk/CHBHPlacementClientProtocol.h>
#import <HeliumSdk/CHBHPlacementClientProtocols.h>
#import <HeliumSdk/CHBHPreBidSettings.h>
#import <HeliumSdk/HeliumDelegates.h>
#import <HeliumSdk/HeliumImpressionData.h>
#import <HeliumSdk/NSError+HeliumErrors.h>

// Forward Swift declarations
@protocol CHBHeliumBannerAdDelegate;
@protocol HeliumInterstitialAd;
@protocol CHBHeliumInterstitialAdDelegate;
@protocol HeliumRewardedAd;
@protocol CHBHeliumRewardedAdDelegate;
@class HeliumBannerView;
@class HeliumInitializationOptions;
typedef NS_ENUM(NSInteger, CHBHBannerSize);

NS_ASSUME_NONNULL_BEGIN

/// Helium SDK entrypoint.
@interface Helium : NSObject

/// Shared instance of the Helium SDK.
/// @returns Shared instance of the Helium SDK.
+ (Helium *)sharedHelium;

#pragma mark - Initialization

/// Initializes the Helium SDK.
/// This method must be called before ads can be served.
/// @param appId Application identifier from the Chartboost dashboard.
/// @param appSignature Application signature from the Chartboost dashboard.
/// @param delegate Optional delegate used to listen for the SDK initialization callback.
- (void)startWithAppId:(NSString *)appId
       andAppSignature:(NSString *)appSignature
              delegate:(nullable id<HeliumSdkDelegate>)delegate __attribute((deprecated("This API will be deprecated in Helium 4.0.0. Use startWithAppId:andAppSignature:options:delegate: instead.")));

/// Initializes the Helium SDK.
/// This method must be called before ads can be served.
/// @param appId Application identifier from the Chartboost dashboard.
/// @param appSignature Application signature from the Chartboost dashboard.
/// @param options Optional initialization options.
/// @param delegate Optional delegate used to listen for the SDK initialization callback.
- (void)startWithAppId:(NSString *)appId
       andAppSignature:(NSString *)appSignature
               options:(nullable HeliumInitializationOptions *)options
              delegate:(nullable id<HeliumSdkDelegate>)delegate;

#pragma mark - Ad Providers

/// Factory method to create a @c HeliumInterstitialAd which will be used to load and show interstitial ads.
/// @param delegate Delegate to receive interstitial ad callbacks.
/// @param placementName Interstitial ad placement from the Chartboost dashboard.
/// @returns The interstitial ad provider if successful; otherwise @c nil will be returned.
- (nullable id<HeliumInterstitialAd>)interstitialAdProviderWithDelegate:(nullable id<CHBHeliumInterstitialAdDelegate>)delegate
                                                       andPlacementName:(NSString *)placementName;

/// Factory method to create a @c HeliumRewardedAd which will be used to load and show rewarded ads.
/// @param delegate Delegate to receive rewarded ad callbacks.
/// @param placementName Rewarded ad placement from the Chartboost dashboard.
/// @returns The rewarded ad provider if successful; otherwise @c nil will be returned.
- (nullable id<HeliumRewardedAd>)rewardedAdProviderWithDelegate:(nullable id<CHBHeliumRewardedAdDelegate>)delegate
                                               andPlacementName:(NSString *)placementName;

/// Factory method to create a @c HeliumBannerView which will be used to load and show banner ads.
/// @param delegate Delegate to receive banner ad callbacks.
/// @param placementName Banner ad placement from the Chartboost dashboard.
/// @param bannerSize Size of the banner to request.
/// @returns The banner ad provider if successful; otherwise @c nil will be returned.
- (nullable HeliumBannerView *)bannerProviderWithDelegate:(nullable id<CHBHeliumBannerAdDelegate>)delegate
                                         andPlacementName:(NSString *)placementName
                                                  andSize:(CHBHBannerSize)bannerSize;

#pragma mark - COPPA and Consent

/// Indicates that the user is subject to COPPA.
/// For more information about COPPA, see: https://answers.chartboost.com/en-us/articles/115001488494
/// @param isSubject User is subject to COPPA.
- (void)setSubjectToCoppa:(BOOL)isSubject;

/// Indicates that the user is subject to GDPR.
/// For more information about GDPR, see: https://answers.chartboost.com/en-us/articles/115001489613
/// @param isSubject User is subject to GDPR.
- (void)setSubjectToGDPR:(BOOL)isSubject;

/// Indicates that the GDPR-applicable user has granted consent to the collection of Personally Identifiable Information.
/// For more information about GDPR, see: https://answers.chartboost.com/en-us/articles/115001489613
/// @param hasGivenConsent GDPR-applicable user has granted consent.
- (void)setUserHasGivenConsent:(BOOL)hasGivenConsent;

/// Indicates that the CCPA-applicable user has granted consent to the collection of Personally Identifiable Information.
/// For more information about CCPA, see: https://answers.chartboost.com/en-us/articles/115001490031
/// @param hasGivenConsent CCPA-applicable user has granted consent.
- (void)setCCPAConsent:(BOOL)hasGivenConsent;

#pragma mark - User Information

/// Optional user identifier sent on every ad request.
@property (nonatomic, copy, nullable) NSString *userIdentifier;

#pragma mark - Game Engine

/// Specifies to the Helium SDK the game engine environment that it is running in.
/// This method should be called before loading ads.
/// @param name Game engine name.
/// @param version Game engine version.
- (void)setGameEngineName:(nullable NSString *)name version:(nullable NSString *)version;

#pragma mark - SDK Information

/// The Helium SDK version.
/// The value is a semantic versioning compliant string.
@property (nonatomic, class, readonly) NSString *sdkVersion;

@end

NS_ASSUME_NONNULL_END
